# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.encoding import smart_text
from markdown2 import markdown as _markdown


def markdown(text, **kwargs):
    return smart_text(_markdown(text, **kwargs).strip())



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from machina.apps.forum_conversation.forum_polls.abstract_models import AbstractTopicPoll
from machina.apps.forum_conversation.forum_polls.abstract_models import AbstractTopicPollOption
from machina.apps.forum_conversation.forum_polls.abstract_models import AbstractTopicPollVote
from machina.core.db.models import model_factory


TopicPoll = model_factory(AbstractTopicPoll)
TopicPollOption = model_factory(AbstractTopicPollOption)
TopicPollVote = model_factory(AbstractTopicPollVote)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from machina.core.app import Application
from machina.core.loading import get_class


class MemberApp(Application):
    name = 'forum_member'

    user_posts_list = get_class('forum_member.views', 'UserPostsView')
    forum_profile_detail_view = get_class('forum_member.views', 'ForumProfileDetailView')
    forum_profile_update_view = get_class('forum_member.views', 'ForumProfileUpdateView')
    topic_subscribe_view = get_class('forum_member.views', 'TopicSubscribeView')
    topic_unsubscribe_view = get_class('forum_member.views', 'TopicUnsubscribeView')
    topic_subscription_list_view = get_class('forum_member.views', 'TopicSubscribtionListView')

    def get_urls(self):
        return [
            # Profile
            url(_(r'^profile/(?P<pk>\d+)/$'),
                self.forum_profile_detail_view.as_view(), name='profile'),
            url(_(r'^profile/(?P<pk>\d+)/posts/$'), self.user_posts_list.as_view(),
                name='user_posts'),
            url(_(r'^profile/edit/$'),
                self.forum_profile_update_view.as_view(), name='profile_update'),

            url(_(r'^subscriptions/$'), self.topic_subscription_list_view.as_view(),
                name='user_subscriptions'),

            url(_(r'^topic/(?P<pk>\d+)/subscribe/$'),
                self.topic_subscribe_view.as_view(), name='topic_subscribe'),
            url(_(r'^topic/(?P<pk>\d+)/unsubscribe/$'),
                self.topic_unsubscribe_view.as_view(), name='topic_unsubscribe'),
        ]


application = MemberApp()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import django.dispatch


forum_moved = django.dispatch.Signal(providing_args=["previous_parent", ])
forum_viewed = django.dispatch.Signal(providing_args=["forum", "user", "request", "response", ])



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.auth import get_user_model
from django.db.models import Q

from machina.conf import settings as machina_settings
from machina.core.db.models import get_model

ForumPermission = get_model('forum_permission', 'ForumPermission')
GroupForumPermission = get_model('forum_permission', 'GroupForumPermission')
UserForumPermission = get_model('forum_permission', 'UserForumPermission')


class ForumPermissionChecker(object):
    """
    The ForumPermissionChecker allows to check forum permissions
    on Forum instances.
    """
    def __init__(self, user):
        self.user = user
        self._forum_perms_cache = {}

    def has_perm(self, perm, forum):
        """
        Checks if the considered user has given permission for the passed forum.
        """
        if not self.user.is_anonymous() and not self.user.is_active:
            # An inactive user cannot have permissions
            return False
        elif self.user and self.user.is_superuser:
            # The superuser have all permissions
            return True
        return perm in self.get_perms(forum)

    def get_perms(self, forum):
        """
        Returns the list of permission codenames of all permissions for the given forum.
        """
        # An inactive user has no permissions
        if not self.user.is_anonymous() and not self.user.is_active:
            return []

        user_model = get_user_model()
        user_groups_related_name = user_model.groups.field.related_query_name()

        if forum.id not in self._forum_perms_cache:
            if self.user and self.user.is_superuser:
                # The superuser has all the permissions
                perms = list(ForumPermission.objects.values_list('codename', flat=True))
            elif self.user:
                default_auth_forum_perms = \
                    machina_settings.DEFAULT_AUTHENTICATED_USER_FORUM_PERMISSIONS

                user_kwargs_filter = {'anonymous_user': True} if self.user.is_anonymous() \
                    else {'user': self.user}

                # Fetches the permissions of the considered user for the given forum
                user_perms = UserForumPermission.objects.select_related() \
                    .filter(**user_kwargs_filter) \
                    .filter(Q(forum__isnull=True) | Q(forum=forum))

                # Computes the list of permissions that are granted for all the forums
                globally_granted_user_perms = list(
                    filter(lambda p: p.has_perm and p.forum is None, user_perms))
                globally_granted_user_perms = [
                    p.permission.codename for p in globally_granted_user_perms]

                # Computes the list of permissions that are granted on a per-forum basis
                per_forum_granted_user_perms = list(
                    filter(lambda p: p.has_perm and p.forum is not None, user_perms))
                per_forum_granted_user_perms = [
                    p.permission.codename for p in per_forum_granted_user_perms]

                # Computes the list of permissions that are not granted on a per-forum basis
                per_forum_nongranted_user_perms = list(
                    filter(lambda p: not p.has_perm and p.forum is not None, user_perms))
                per_forum_nongranted_user_perms = [
                    p.permission.codename for p in per_forum_nongranted_user_perms]

                # If the considered user have no global permissions, the permissions defined by
                # the DEFAULT_AUTHENTICATED_USER_FORUM_PERMISSIONS settings are used instead
                if self.user.is_authenticated() and not globally_granted_user_perms:
                    globally_granted_user_perms = default_auth_forum_perms

                # Finally computes the list of permission codenames that are granted to
                # the user for the considered forum
                granted_user_perms = [
                    c for c in globally_granted_user_perms if
                    c not in per_forum_nongranted_user_perms] + per_forum_granted_user_perms
                granted_user_perms = set(granted_user_perms)

                perms = granted_user_perms

                # If the user is a registered user, we have to check the permissions
                # of its groups in order to determine the additional permissions he could
                # have
                if not self.user.is_anonymous():
                    group_perms = GroupForumPermission.objects.select_related() \
                        .filter(**{'group__{}'.format(user_groups_related_name): self.user}) \
                        .filter(Q(forum__isnull=True) | Q(forum=forum))

                    globally_granted_group_perms = list(
                        filter(lambda p: p.has_perm and p.forum is None, group_perms))
                    globally_granted_group_perms = [
                        p.permission.codename for p in globally_granted_group_perms]

                    per_forum_granted_group_perms = list(
                        filter(lambda p: p.has_perm and p.forum is not None, group_perms))
                    per_forum_granted_group_perms = [
                        p.permission.codename for p in per_forum_granted_group_perms]

                    per_forum_nongranted_group_perms = list(
                        filter(lambda p: not p.has_perm and p.forum is not None, group_perms))
                    per_forum_nongranted_group_perms = [
                        p.permission.codename for p in per_forum_nongranted_group_perms]

                    granted_group_perms = [
                        c for c in globally_granted_group_perms if
                        c not in per_forum_nongranted_group_perms] + per_forum_granted_group_perms
                    granted_group_perms = filter(
                        lambda x: x not in per_forum_nongranted_user_perms, granted_group_perms)
                    granted_group_perms = set(granted_group_perms)

                    # Includes the permissions granted for the user' groups in the initial set of
                    # permission codenames
                    perms |= granted_group_perms

            self._forum_perms_cache[forum.id] = perms

        return self._forum_perms_cache[forum.id]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import VERSION as DJANGO_VESION
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _


# PILImage
try:
    # Try from the Pillow (or one variant of PIL) install location first.
    from PIL import Image as PILImage
except ImportError as err:  # pragma: no cover
    try:
        # If that failed, try the alternate import syntax for PIL.
        import Image as PILImage  # noqa
    except ImportError as err:
        # Neither worked, so it's likely not installed.
        raise ImproperlyConfigured(
            _('Neither Pillow nor PIL could be imported: {}').format(err)
        )


# Slugify with 'allow_unicode' option
if DJANGO_VESION < (1, 9):
    import re
    import unicodedata

    from django.utils import six
    from django.utils.encoding import force_text
    from django.utils.functional import allow_lazy
    from django.utils.safestring import mark_safe
    from django.utils.safestring import SafeText

    # Orginally comes from https://github.com/django/django/blob/1.9/django/utils/text.py#L413-L427
    # for compatibility reasons.
    def slugify(value, allow_unicode=False):
        """
        Convert to ASCII if 'allow_unicode' is False. Convert spaces to hyphens.
        Remove characters that aren't alphanumerics, underscores, or hyphens.
        Convert to lowercase. Also strip leading and trailing whitespace.
        """
        value = force_text(value)
        if allow_unicode:
            value = unicodedata.normalize('NFKC', value)
            value = re.sub('[^\w\s-]', '', value, flags=re.U).strip().lower()
            return mark_safe(re.sub('[-\s]+', '-', value, flags=re.U))
        value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
        value = re.sub('[^\w\s-]', '', value).strip().lower()
        return mark_safe(re.sub('[-\s]+', '-', value))
    slugify = allow_lazy(slugify, six.text_type, SafeText)
else:
    from django.utils.text import slugify


# MiddlewareMixin
try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:  # pragma: no cover
    class MiddlewareMixin(object):
        pass



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class FeedsRegistryConfig(AppConfig):
    label = 'feeds'
    name = 'machina.apps.forum_feeds'
    verbose_name = _('Machina: Forum feeds')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class MemberRegistryConfig(AppConfig):
    label = 'forum_member'
    name = 'machina.apps.forum_member'
    verbose_name = _('Machina: Forum members')

    def ready(self):  # pragma: no cover
        from . import receivers  # noqa
